@@ -190,7 +190,7 @@ asGDALROD_SGDF <- function(from) {
 	gt = .Call('RGDAL_GetGeoTransform', x, PACKAGE="rgdal")
         if (attr(gt, "CE_Failure")) warning("GeoTransform values not available")
 	if (any(gt[c(3,5)] != 0.0)) stop("Diagonal grid not permitted")
-	data = getRasterData(x)
+	data = getRasterData(x, list_out=TRUE)
 	cellsize = abs(c(gt[2],gt[6]))
 	ysign <- sign(gt[6])
 	co.x <- gt[1] + (offset[2] + half.cell[2]) * cellsize[1]
@@ -200,17 +200,18 @@ asGDALROD_SGDF <- function(from) {
 		abs(cellsize[2]))
 	cellcentre.offset <- c(x=co.x, y=co.y)
 	grid = GridTopology(cellcentre.offset, cellsize, rev(output.dim))
-	if (length(d) == 2L)
-		df = list(band1 = as.vector(data))
-	else {
-		df <- vector(mode="list", length=d[3])
-		df[[1]] <- as.vector(data[,,1, drop = FALSE])
-		for (band in 2:d[3])
-			df[[band]] <- as.vector(data[,,band, drop = FALSE])
-		names(df) = paste("band", 1:d[3], sep="")
-	}
+#	if (length(d) == 2L)
+#		df = list(band1 = as.vector(data))
+#	else {
+#		df <- vector(mode="list", length=d[3])
+#		df[[1]] <- as.vector(data[,,1, drop = FALSE])
+#		for (band in 2:d[3])
+#			df[[band]] <- as.vector(data[,,band, drop = FALSE])
+#		names(df) = paste("band", 1:d[3], sep="")
+#	}
 	return(SpatialGridDataFrame(grid = grid, 
-		data = data.frame(df), proj4string=CRS(p4s)))
+		data = as.data.frame(data), proj4string=CRS(p4s)))
+#		data = data.frame(df), proj4string=CRS(p4s)))
 }
 
 setAs("GDALReadOnlyDataset", "SpatialGridDataFrame", asGDALROD_SGDF)
@@ -237,7 +238,7 @@ asSGDF_GROD <- function(x, offset, region.dim, output.dim, p4s=NULL, ..., half.c
         if (attr(gt, "CE_Failure")) warning("GeoTransform values not available")
 	if (any(gt[c(3,5)] != 0.0)) stop("Diagonal grid not permitted")
 	data = getRasterData(x, offset=offset, 
-		region.dim=region.dim, output.dim=output.dim, ...)
+		region.dim=region.dim, output.dim=output.dim, ..., list_out=TRUE)
 	if (!odim_flag) cellsize = abs(c(gt[2],gt[6]))
 	else {
 		icellsize = abs(c(gt[2],gt[6]))
@@ -252,16 +253,17 @@ asSGDF_GROD <- function(x, offset, region.dim, output.dim, p4s=NULL, ..., half.c
 		abs(cellsize[2]))
 	cellcentre.offset <- c(x=co.x, y=co.y)
 	grid = GridTopology(cellcentre.offset, cellsize, rev(output.dim))
-	if (length(d) == 2L)
-		df = list(band1 = as.vector(data))
-	else {
-		df <- vector(mode="list", length=d[3])
-		df[[1]] <- as.vector(data[,,1, drop = FALSE])
-		for (band in 2:d[3])
-			df[[band]] <- as.vector(data[,,band, drop = FALSE])
-		names(df) = paste("band", 1:d[3], sep="")
-	}
-	df1 <- data.frame(df)
+#	if (length(d) == 2L)
+#		df = list(band1 = as.vector(data))
+#	else {
+#		df <- vector(mode="list", length=d[3])
+#		df[[1]] <- as.vector(data[,,1, drop = FALSE])
+#		for (band in 2:d[3])
+#			df[[band]] <- as.vector(data[,,band, drop = FALSE])
+#		names(df) = paste("band", 1:d[3], sep="")
+#	}
+#	df1 <- data.frame(df)
+	df1 <- as.data.frame(data)
 	data = SpatialGridDataFrame(grid = grid, 
 		data = df1, proj4string=CRS(p4s))
 	return(data)
@@ -298,6 +300,8 @@ readGDAL = function(fname, offset, region.dim, output.dim, band, p4s=NULL, ...,
 	gt = .Call('RGDAL_GetGeoTransform', x, PACKAGE="rgdal")
         if (attr(gt, "CE_Failure")) warning("GeoTransform values not available")
 	# [1] 178400     40      0 334000      0    -40
+        opSilent <- get("silent", envir=.RGDAL_CACHE)
+        assign("silent", silent, envir=.RGDAL_CACHE)
 	if (any(gt[c(3,5)] != 0.0)) {
 		data = getRasterTable(x, band=band, offset=offset, 
 			region.dim=region.dim, ...)
@@ -334,27 +338,28 @@ readGDAL = function(fname, offset, region.dim, output.dim, band, p4s=NULL, ...,
 			rev(output.dim))
 #			rev(region.dim))
 		data = getRasterData(x, band=band, offset=offset, 
-			region.dim=region.dim, output.dim=output.dim, ...)
+			region.dim=region.dim, output.dim=output.dim, ..., list_out=TRUE)
 		GDAL.close(x)
-		if (length(d) == 2L)
-			df = list(band1 = as.vector(data))
-		else {
-			df <- vector(mode="list", length=d[3])
-			df[[1]] <- as.vector(data[,,1, drop = FALSE])
-			for (band in 2:d[3])
-				df[[band]] <- as.vector(data[,,band, drop = FALSE])
+#		if (length(d) == 2L)
+#			df = list(band1 = as.vector(data))
+#		else {
+#			df <- vector(mode="list", length=d[3])
+#			df[[1]] <- as.vector(data[,,1, drop = FALSE])
+#			for (band in 2:d[3])
+#				df[[band]] <- as.vector(data[,,band, drop = FALSE])
 #			df = as.data.frame(df)
-			names(df) = paste("band", 1:d[3], sep="")
-		}
+#			names(df) = paste("band", 1:d[3], sep="")
+#		}
 		data = SpatialGridDataFrame(grid = grid, 
-			data = data.frame(df), proj4string=CRS(p4s))
+			data = as.data.frame(data), proj4string=CRS(p4s))
 	}
+        assign("silent", opSilent, envir=.RGDAL_CACHE)
 	return(data)
 }
 
 writeGDAL = function(dataset, fname, drivername = "GTiff", type = "Float32", 
 		mvFlag = NA, options=NULL, copy_drivername = "GTiff",
-                setStatistics=FALSE)
+                setStatistics=FALSE, colorTables=NULL, catNames=NULL)
 {
 	if (nchar(fname) == 0) stop("empty file name")
         x <- gdalDrivers()
@@ -362,7 +367,8 @@ writeGDAL = function(dataset, fname, drivername = "GTiff", type = "Float32",
         if (drivername %in% copy_only) {
 	    tds.create <- create2GDAL(dataset=dataset,
                 drivername=copy_drivername, type=type,
-                mvFlag=mvFlag, fname=NULL, setStatistics=setStatistics)
+                mvFlag=mvFlag, fname=NULL, setStatistics=setStatistics,
+                colorTables=colorTables, catNames=catNames)
             tds.copy <- copyDataset(tds.create, driver=drivername, fname=fname)
             GDAL.close(tds.create)
 	    saveDataset(tds.copy, fname, options=options)
@@ -371,7 +377,8 @@ writeGDAL = function(dataset, fname, drivername = "GTiff", type = "Float32",
         } else {
 	    tds.out <- create2GDAL(dataset=dataset, drivername=drivername, 
 		type=type, mvFlag=mvFlag, options=options, fname=fname,
-                setStatistics=setStatistics)
+                setStatistics=setStatistics,  colorTables=colorTables,
+                catNames=catNames)
 	    saveDataset(tds.out, fname, options=options)
 # RSB 120921
             GDAL.close(tds.out)
@@ -382,7 +389,7 @@ writeGDAL = function(dataset, fname, drivername = "GTiff", type = "Float32",
 	invisible(fname)
 }
 
-create2GDAL = function(dataset, drivername = "GTiff", type = "Float32", mvFlag = NA, options=NULL, fname=NULL, setStatistics=FALSE)
+create2GDAL = function(dataset, drivername = "GTiff", type = "Float32", mvFlag = NA, options=NULL, fname=NULL, setStatistics=FALSE, colorTables=NULL, catNames=NULL)
 {
 	stopifnot(gridded(dataset))
 	fullgrid(dataset) = TRUE
@@ -418,6 +425,18 @@ create2GDAL = function(dataset, drivername = "GTiff", type = "Float32", mvFlag =
             if (getDriverName(getDriver(tds.out)) == "RST") 
                 stop("RST files must have a valid coordinate reference system")
         }
+        if (!is.null(colorTables)) {
+            stopifnot(is.list(colorTables))
+            stopifnot(length(colorTables) == nbands)
+            if (type != "Byte") {
+#                colorTables <- NULL
+                warning("colorTables valid for Byte type only in some drivers")
+            }
+        }
+        if (!is.null(catNames)) {
+            stopifnot(is.list(catNames))
+            stopifnot(length(catNames) == nbands)
+        }
 	for (i in 1:nbands) {
 		band = as.matrix(dataset[i])
 		if (!is.numeric(band)) stop("Numeric bands required")
@@ -429,20 +448,31 @@ create2GDAL = function(dataset, drivername = "GTiff", type = "Float32", mvFlag =
 		if (!is.na(mvFlag))
 			band[is.na(band)] = mvFlag
 		putRasterData(tds.out, band, i)
+                tds.out_b <- getRasterBand(dataset=tds.out, band=i)
 		if (!is.na(mvFlag)) {
-		    tds.out_b <- getRasterBand(dataset=tds.out, band=i)
 		    .Call("RGDAL_SetNoDataValue", tds.out_b, as.double(mvFlag),
 		        PACKAGE = "rgdal")
 		}
                 if (setStatistics) {
-                    tds.out_b <- getRasterBand(dataset=tds.out, band=i)
-                    .Call("RGDAL_SetStatistics", tds.out_b,
-                        as.double(statistics), PACKAGE = "rgdal")
+                    .gd_SetStatistics(tds.out_b, as.double(statistics))
+                }
+                if (!is.null(colorTables)) {
+                    icT <- colorTables[[i]]
+                    if (!is.null(icT)) {
+                        .gd_SetRasterColorTable(tds.out_b, icT)
+                    }
+                }
+                if (!is.null(catNames)) {
+                    icN <- catNames[[i]]
+                    if (!is.null(icN)) {
+                        .gd_SetCategoryNames(tds.out_b, icN)
+                    }       
                 }
 	}
 	tds.out
 }
 
+
 gdalDrivers <- function() getGDALDriverNames()
 
 toSigned <- function(x, base) {
